# ANTS-Hub - Copyright (C) Entr'ouvert

import datetime
import zoneinfo

import pytest
import responses
import responses.matchers

from ants_hub.data.models import Collectivite, Config, Lieu, Plage, Raccordement, RendezVous


@pytest.fixture(autouse=True)
def setup(freezer):
    freezer.move_to('2023-03-03T12:00:00+02:00')


def test_ping(django_app, db):
    django_app.get('/api/chrono/ping/', status=401)
    django_app.get('/api/chrono/ping/', headers={'Authorization': ''}, status=401)
    django_app.get('/api/chrono/ping/', headers={'Authorization': 'Basic abc'}, status=401)
    django_app.set_authorization(('Basic', ('abcd', '')))
    django_app.get('/api/chrono/ping/', status=401)
    Raccordement.objects.create(name='plateforme', apikey='abcd')
    assert django_app.get('/api/chrono/ping/').json == {'err': 0}


def test_rendez_vous_disponibles(django_app, db):
    Raccordement.objects.create(name='plateforme', apikey='abcd')
    django_app.set_authorization(('Basic', ('abcd', '')))
    assert django_app.get('/api/chrono/rendez-vous-disponibles/').json == {
        'err': 0,
        'collectivites': [],
    }
    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'logo_url': 'https://saint-didier.fr/logo.png',
                }
            ]
        },
    )
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 1,
            'collectivites_updated': 0,
            'lieux_created': 0,
            'lieux_updated': 0,
            'lieux_deleted': 0,
            'plage_created': 0,
            'plage_updated': 0,
            'plage_deleted': 0,
            'rdv_created': 0,
            'rdv_updated': 0,
            'rdv_deleted': 0,
        },
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv2/',
                    'logo_url': 'https://saint-didier.fr/logo.png',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                        }
                    ],
                }
            ]
        },
    )
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 0,
            'collectivites_updated': 1,
            'lieux_created': 1,
            'lieux_updated': 0,
            'lieux_deleted': 0,
            'plage_created': 0,
            'plage_updated': 0,
            'plage_deleted': 0,
            'rdv_created': 0,
            'rdv_updated': 0,
            'rdv_deleted': 0,
        },
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                }
                            ],
                        }
                    ],
                }
            ]
        },
    )
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 0,
            'collectivites_updated': 0,
            'lieux_created': 0,
            'lieux_updated': 0,
            'lieux_deleted': 0,
            'plage_created': 2,
            'plage_updated': 0,
            'plage_deleted': 0,
            'rdv_created': 0,
            'rdv_updated': 0,
            'rdv_deleted': 0,
        },
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['PASSPORT'],
                                }
                            ],
                        }
                    ],
                }
            ]
        },
    )
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 0,
            'collectivites_updated': 0,
            'lieux_created': 0,
            'lieux_updated': 0,
            'lieux_deleted': 0,
            'plage_created': 0,
            'plage_updated': 0,
            'plage_deleted': 1,
            'rdv_created': 0,
            'rdv_updated': 0,
            'rdv_deleted': 0,
        },
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'rdvs': [
                                {
                                    'id': 'abcd',
                                    'date': '2023-03-20T15:00:00+02:00',
                                }
                            ],
                        }
                    ],
                }
            ]
        },
    )
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 0,
            'collectivites_updated': 0,
            'lieux_created': 0,
            'lieux_updated': 0,
            'lieux_deleted': 0,
            'plage_created': 0,
            'plage_updated': 0,
            'plage_deleted': 0,
            'rdv_created': 1,
            'rdv_updated': 0,
            'rdv_deleted': 0,
        },
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'rdvs': [
                                {
                                    'id': 'abcd',
                                    'date': '2023-03-20T15:00:00+02:00',
                                    'annule': True,
                                }
                            ],
                        }
                    ],
                }
            ]
        },
    )
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 0,
            'collectivites_updated': 0,
            'lieux_created': 0,
            'lieux_updated': 0,
            'lieux_deleted': 0,
            'plage_created': 0,
            'plage_updated': 0,
            'plage_deleted': 0,
            'rdv_created': 0,
            'rdv_updated': 0,
            'rdv_deleted': 1,
        },
    }

    response = django_app.post('/api/chrono/rendez-vous-disponibles/', params=b'xxx', status=400)
    assert response.json['err'] == 'invalid-json'

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'rdvs': [
                                {
                                    'id': 'abcd',
                                }
                            ],
                        }
                    ],
                }
            ]
        },
        status=400,
    )
    assert response.json['err'] == 'invalid-json'

    assert django_app.get('/api/chrono/rendez-vous-disponibles/').json == {
        'err': 0,
        'collectivites': [
            {
                'id': 'col1',
                'nom': 'Saint-Didier',
                'url': 'https://saint-didier.fr/rdv2/',
                'logo_url': 'https://saint-didier.fr/logo.png',
                'created': '2023-03-03T10:00:00+00:00',
                'last_update': '2023-03-03T10:00:00+00:00',
                'lieux': [
                    {
                        'id': 'lieu1',
                        'latitude': 2.3,
                        'longitude': 1.5,
                        'nom': 'Mairie de Saint-Didier',
                        'numero_rue': '2 rue du four',
                        'code_postal': '99999',
                        'ville': 'Saint-Didier',
                        'nombre_de_jours_avec_rdv_disponibles': 1,
                        'nombre_de_pre_demandes_actives': 0,
                    }
                ],
                'nombre_de_jours_avec_rdv_disponibles': 1,
                'nombre_de_lieux': 1,
                'nombre_de_pre_demandes_actives': 0,
            }
        ],
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '15:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                }
                            ],
                        }
                    ],
                }
            ]
        },
        status=400,
    )
    assert response.json == {
        'err': 1,
        'error': '["plage {\'date\': \'2023-03-20\', \'types_rdv\': [\'CNI\', '
        "'PASSPORT'], 'heure_debut': '15:00+02:00', 'heure_fin': "
        "'12:00+02:00', 'duree': 15, 'personnes': 1}: heure de fin "
        'inférieure à heure de début"]',
    }


@responses.activate
def test_predemandes(db, django_app):
    Config.set(Config.REQUEST_TO_ANTS_AUTH_TOKEN, 'xyz')
    Raccordement.objects.create(name='plateforme', apikey='abcd')
    django_app.set_authorization(('Basic', ('abcd', '')))

    document = {
        'xyz': [
            {
                'meeting_point': 'MAIRIE DE SAINT-DIDIER',
                'datetime': '2023-04-03T10:00:00Z',
                'management_url': 'https://saint-didier/rdv/1/',
                'cancel_url': 'https://saint-didier/rdv/1/cancel/',
            }
        ]
    }
    rdv_api_url = 'https://rendez-vous-api.france-identite.fr/api/searchApplicationIds'
    responses.add(
        responses.GET,
        rdv_api_url,
        json=document,
        status=200,
        match=[responses.matchers.header_matcher({'x-hub-rdv-auth-token': 'xyz'})],
    )

    response = django_app.get('/api/chrono/predemandes/', params={'identifiant_predemande': 'xyz'})
    assert response.json['err'] == 0
    assert response.json['data'] == [
        {
            'meeting_point': 'MAIRIE DE SAINT-DIDIER',
            'datetime': '2023-04-03T10:00:00',
            'management_url': 'https://saint-didier/rdv/1/',
            'cancel_url': 'https://saint-didier/rdv/1/cancel/',
        }
    ]
    assert (
        responses._default_mock.calls[0].request.url
        == 'https://rendez-vous-api.france-identite.fr/api/searchApplicationIds?application_ids=xyz'
    )


def test_rendez_vous_disponibles_full(django_app, db, freezer):
    Raccordement.objects.create(name='plateforme', apikey='abcd')
    django_app.set_authorization(('Basic', ('abcd', '')))
    assert django_app.get('/api/chrono/rendez-vous-disponibles/').json == {
        'err': 0,
        'collectivites': [],
    }
    assert Collectivite.objects.count() == 0
    assert Lieu.objects.count() == 0
    assert Plage.objects.count() == 0
    assert RendezVous.objects.count() == 0

    django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-21',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-22',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                            'rdvs': [
                                {
                                    'id': 'abcd1',
                                    'date': '2023-03-23T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd2',
                                    'date': '2023-03-24T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd3',
                                    'date': '2023-03-25T15:00:00+02:00',
                                },
                            ],
                        },
                        {
                            'id': 'lieu2',
                            'nom': 'Mairie annexe de Saint-Didier',
                            'numero_rue': '3 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-21',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-22',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                            'rdvs': [
                                {
                                    'id': 'abcd4',
                                    'date': '2023-03-17T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd5',
                                    'date': '2023-03-18T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd6',
                                    'date': '2023-03-19T15:00:00+02:00',
                                },
                            ],
                        },
                    ],
                }
            ]
        },
    )

    assert Collectivite.objects.count() == 1
    assert Lieu.objects.count() == 2
    assert Plage.objects.count() == 12
    assert RendezVous.objects.count() == 6
    plage_last_update = set(Plage.objects.values_list('last_update', flat=True))
    rendez_vous_last_update = set(RendezVous.objects.values_list('last_update', flat=True))

    # advance clock to verify last_update times
    freezer.tick()
    django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-22',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                            'rdvs': [
                                {
                                    'id': 'abcd1',
                                    'date': '2023-03-23T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd3',
                                    'date': '2023-03-25T15:00:00+02:00',
                                },
                            ],
                        },
                        {
                            'id': 'lieu2',
                            'nom': 'Mairie annexe de Saint-Didier',
                            'numero_rue': '3 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-22',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                            'rdvs': [
                                {
                                    'id': 'abcd4',
                                    'date': '2023-03-17T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd6',
                                    'date': '2023-03-19T15:00:00+02:00',
                                },
                            ],
                        },
                    ],
                }
            ]
        },
    )

    assert Collectivite.objects.count() == 1
    assert Lieu.objects.count() == 2
    assert Plage.objects.count() == 12
    assert RendezVous.objects.count() == 6
    # check objects are not updated/created uselessly
    assert plage_last_update == set(Plage.objects.values_list('last_update', flat=True))
    assert rendez_vous_last_update == set(RendezVous.objects.values_list('last_update', flat=True))

    # advance clock to verify last_update times
    freezer.tick()
    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'full': True,
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'full': True,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-22',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                            'rdvs': [
                                {
                                    'id': 'abcd1',
                                    'date': '2023-03-23T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd3',
                                    'date': '2023-03-25T15:00:00+02:00',
                                },
                            ],
                        },
                    ],
                }
            ]
        },
    )

    assert Collectivite.objects.count() == 1
    assert Lieu.objects.count() == 1
    assert Plage.objects.count() == 4
    assert RendezVous.objects.count() == 2
    assert set(RendezVous.objects.values_list('identifiant_predemande', flat=True)) == {'abcd1', 'abcd3'}
    # check objects are not updated/created uselessly
    assert plage_last_update >= set(Plage.objects.values_list('last_update', flat=True))
    assert rendez_vous_last_update >= set(RendezVous.objects.values_list('last_update', flat=True))
    assert response.json == {
        'err': 0,
        'data': {
            'collectivites_created': 0,
            'collectivites_updated': 0,
            'lieux_created': 0,
            'lieux_updated': 0,
            'lieux_deleted': 1,
            'plage_created': 0,
            'plage_updated': 0,
            'plage_deleted': 8,
            'rdv_created': 0,
            'rdv_updated': 0,
            'rdv_deleted': 4,
        },
    }

    django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'full': True,
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'full': True,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-22',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '12:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                            'rdvs': [
                                {
                                    'id': 'abcd1',
                                    'date': '2023-03-23T15:00:00+02:00',
                                },
                                {
                                    'id': 'abcd3',
                                    'date': '2023-03-25T15:00:00+02:00',
                                },
                            ],
                        },
                    ],
                }
            ]
        },
    )


def test_naive_time(django_app, db, freezer):
    Raccordement.objects.create(name='plateforme', apikey='abcd')
    django_app.set_authorization(('Basic', ('abcd', '')))
    assert django_app.get('/api/chrono/rendez-vous-disponibles/').json == {
        'err': 0,
        'collectivites': [],
    }

    django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI'],
                                    'heure_debut': '08:00',
                                    'heure_fin': '09:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                        },
                    ],
                }
            ]
        },
    )

    assert Plage.objects.count() == 1
    plage = Plage.objects.get()

    # checks that times from the plage update were interpreted as naive
    # datetime in the Europe/Paris timezone
    assert list(x for x, y in plage.available_time_slots()) == [
        datetime.datetime(2023, 3, 20, 8, 0, tzinfo=zoneinfo.ZoneInfo(key='Europe/Paris')),
        datetime.datetime(2023, 3, 20, 8, 15, tzinfo=zoneinfo.ZoneInfo(key='Europe/Paris')),
        datetime.datetime(2023, 3, 20, 8, 30, tzinfo=zoneinfo.ZoneInfo(key='Europe/Paris')),
        datetime.datetime(2023, 3, 20, 8, 45, tzinfo=zoneinfo.ZoneInfo(key='Europe/Paris')),
    ]


def test_plages_ordering(django_app, db, freezer):
    Raccordement.objects.create(name='plateforme', apikey='abcd')

    django_app.set_authorization(('Basic', ('abcd', '')))
    django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv/',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'plages': [
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '12:00+02:00',
                                    'heure_fin': '13:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '08:00+02:00',
                                    'heure_fin': '09:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                                {
                                    'date': '2023-03-20',
                                    'types_rdv': ['CNI', 'PASSPORT'],
                                    'heure_debut': '10:00+02:00',
                                    'heure_fin': '11:00+02:00',
                                    'duree': 15,
                                    'personnes': 1,
                                },
                            ],
                        },
                    ],
                }
            ]
        },
    )


def rdv_response(response):
    return {key: value for key, value in response.json.get('data', {}).items() if value > 0}


class TestUpdate:
    @pytest.fixture(autouse=True)
    def setup(self, django_app, db):
        Raccordement.objects.create(name='plateforme', apikey='abcd')
        django_app.set_authorization(('Basic', ('abcd', '')))

    def test_collectivite_and_lieu_update(self, django_app, db):
        response = django_app.post_json(
            '/api/chrono/rendez-vous-disponibles/',
            params={
                'collectivites': [
                    {
                        'id': 'col1',
                        'nom': 'Saint-Didier',
                        'url': 'https://saint-didier.fr/rdv/',
                        'logo_url': 'https://saint-didier.fr/logo.png',
                        'lieux': [
                            {
                                'id': 'lieu1',
                                'nom': 'Mairie de Saint-Didier',
                                'numero_rue': '2 rue du four',
                                'code_postal': '99999',
                                'ville': 'Saint-Didier',
                                'longitude': 1.5,
                                'latitude': 2.3,
                            }
                        ],
                    }
                ]
            },
        )

        assert rdv_response(response) == {
            'collectivites_created': 1,
            'lieux_created': 1,
        }

        response = django_app.post_json(
            '/api/chrono/rendez-vous-disponibles/',
            params={
                'collectivites': [
                    {
                        'id': 'col1',
                        'nom': 'Saint-François',
                        'url': 'https://saint-francois.fr/rdv/',
                        'logo_url': 'https://saint-didier.fr/logo.png',
                        'lieux': [
                            {
                                'id': 'lieu1',
                                'nom': 'Mairie de Saint-François',
                                'numero_rue': '2 rue du four',
                                'code_postal': '99999',
                                'ville': 'Saint-François',
                                'longitude': 1.5,
                                'latitude': 2.3,
                            }
                        ],
                    }
                ]
            },
        )

        assert rdv_response(response) == {
            'collectivites_updated': 1,
            'lieux_updated': 1,
        }


def test_annulation_rdv_full(django_app, db):
    Raccordement.objects.create(name='plateforme', apikey='abcd')
    django_app.set_authorization(('Basic', ('abcd', '')))

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'nom': 'Saint-Didier',
                    'url': 'https://saint-didier.fr/rdv2/',
                    'logo_url': 'https://saint-didier.fr/logo.png',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'nom': 'Mairie de Saint-Didier',
                            'numero_rue': '2 rue du four',
                            'code_postal': '99999',
                            'ville': 'Saint-Didier',
                            'longitude': 1.5,
                            'latitude': 2.3,
                            'full': True,
                            'rdvs': [
                                {
                                    'id': 'abcd',
                                    'date': '2023-03-20T15:00:00+02:00',
                                }
                            ],
                        }
                    ],
                }
            ]
        },
    )
    assert rdv_response(response) == {
        'collectivites_created': 1,
        'lieux_created': 1,
        'rdv_created': 1,
    }

    response = django_app.post_json(
        '/api/chrono/rendez-vous-disponibles/',
        params={
            'collectivites': [
                {
                    'id': 'col1',
                    'lieux': [
                        {
                            'id': 'lieu1',
                            'full': True,
                            'rdvs': [
                                {
                                    'id': 'abcd',
                                    'date': '2023-03-20T15:00:00+02:00',
                                    'annule': True,
                                }
                            ],
                        }
                    ],
                }
            ]
        },
    )
    assert rdv_response(response) == {
        'rdv_deleted': 1,
    }
